//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: triclops.h 309688 2016-12-21 18:28:11Z silvanoa $
//=============================================================================

//=============================================================================
/** @file triclops.h
 *  @brief Triclops operations API */
//=============================================================================

#ifndef TRICLOPS_H
#define TRICLOPS_H

//=============================================================================
// Project includes
//=============================================================================
#include <triclopsenum.h>
#include <triclopstype.h>
#include <triclops3d.h>
#include <triclopsbuffer.h>
#include <triclopscontext.h>
#include <triclopsvalidation.h>
#include <triclopsrectify.h>
#include <triclopsstereo.h>
#include <triclopsimageio.h>

#ifdef __cplusplus
extern "C"
{
#endif

/** @defgroup TriclopsGeneral General Triclops functions
 *
 * General purpose functions regarding the library, such as current version retrieval.
 */

/** @addtogroup TriclopsGeneral
 *
 * @{
 */ // start of group TriclopsGeneral

//=============================================================================
// General Triclops Functions
//=============================================================================

/** @brief Retrieves the current version of the Triclops SDK
 *
 *  This function returns internally-handled memory.  The caller should not
 *  free the returned pointer.
 *
 * @retval A string with the Triclops SDK version (e.g. "4.0.0.0")
 *
 * @see triclopsGetVersion()
 */
const char *
triclopsVersion();

/** @brief Retrieves the current version of the Triclops SDK
 *
 * The function performs a similar operation to that of triclopsVersion(),
 * but retrieves the version to a dedicated structure.
 *
 * @param pVersion pointer to the structure to receive the library version.
 *
 * @return An error code representing the outcome of the function
 * @retval TriclopsErrorOk              The function completed successfully
 * @retval TriclopsErrorInvalidVersion  The provided version structure pointer is NULL
 *
 * @see triclopsVersion()
 */
enum TriclopsError
triclopsGetVersion(TriclopsVersion *pVersion);


/** @} */ // end of group TriclopsGeneral



#ifdef __cplusplus
}
#endif

#endif  // #ifndef TRICLOPS_H
