//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: grab_stereo.h 304806 2016-11-09 16:10:31Z micheleb $
//=============================================================================


#ifndef GRAB_STEREO_EXAMPLE_H
#define GRAB_STEREO_EXAMPLE_H

//=============================================================================
// System Includes
//=============================================================================
#include <string>

//=============================================================================
// PGR Includes
//=============================================================================
#include "triclops.h"
#include "fc2triclops.h"

//
// aliases namespaces
//
namespace FC2 = FlyCapture2;
namespace FC2T = Fc2Triclops;

// struct containing image needed for processing
struct ImageContainer
{
    FC2::Image tmp[2];
    FC2::Image unprocessed[2];
};

enum IMAGE_SIDE
{
    RIGHT = 0, LEFT
};

// configue camera to capture image
void configureCamera( FC2::Camera &camera );

// return if the connected camera is a color camera
bool isColorCamera(FC2::Camera & camera);

// generate Triclops context from connected camera
void generateTriclopsContext( FC2::Camera & camera,
                         TriclopsContext & context );

// capture image from connected camera
void grabImage (  FC2::Camera & camera,
                 FC2::Image  & grabbedImage );

// generate color stereo input
void generateColorStereoInput( TriclopsContext const  &context,
                         FC2::Image const       &grabbedImage,
                         ImageContainer         &imageCont,
                         TriclopsColorStereoPair &stereoPair );

                         // generate mono stereo input
void generateMonoStereoInput( TriclopsContext const  &context,
                         FC2::Image const       &grabbedImage,
                         ImageContainer         &imageCont,
                         TriclopsMonoStereoPair &stereoPair );

// carry out stereo processing pipeline
void doStereo(TriclopsContext const & triclops, const std::string &_disparityFile);

void handleError(const std::string &description, const FlyCapture2::Error &error, int lineNumber);
void handleError(const std::string &description, const Fc2Triclops::ErrorType &error, int lineNumber);

#endif //GRAB_STEREO_EXAMPLE_H
