//=============================================================================
// Copyright © 2016 FLIR Integrated Imaging Solutions, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of FLIR
// Integrated Imaging Solutions, Inc. ("Confidential Information"). You
// shall not disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with FLIR Integrated Imaging Solutions, Inc. (FLIR).
//
// FLIR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. FLIR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: Example_Exception.cpp 304806 2016-11-09 16:10:31Z micheleb $
//=============================================================================

//=============================================================================
// Example Includes
//=============================================================================
#include "Example_Exception.h"

//=============================================================================
// System Includes
//=============================================================================
#include <sstream>
#include <iostream>

namespace triclops_examples {

Example_Exception::Example_Exception(const std::string &description, const std::string &status, int lineNumber) :
        function_name(description), triclops_Status(status), line(lineNumber)
{
    std::ostringstream error;

    error << "Triclops Error: " << triclops_Status;
    error << " (line " << line << ")";
    error << std::endl;
    error << "Function called: " << function_name << std::endl;

    error_log = error.str();
}

const char *Example_Exception::what() const noexcept
{
    return error_log.c_str();
}


void handleError(const std::string &description, TriclopsError status, int lineNumber)
{
    if (status != TriclopsErrorOk) {
        throw Example_Exception(description, triclopsErrorToString(status), lineNumber);
    }
}

} // namespace triclops_examples
