//=============================================================================
// Copyright © 2015 Point Grey Research, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Point Grey Research, Inc. (PGR).
//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: ContextManager.cpp 287944 2016-06-23 15:49:35Z micheleb $
//=============================================================================

//=============================================================================
// Project Includes
//=============================================================================
#include "ContextManager.h"
#include "OfflineContext.h"

//=============================================================================
// Qt Includes
//=============================================================================
#include <QHash>
#include <QString>
#include <QDebug>

namespace TriclopsDemo {

ContextManager::ContextManager()
{

}

ContextManager::~ContextManager()
{
    if (!contextHash.isEmpty()){
        QHash<QString, TriclopsDemo::ViewContext*>::const_iterator contextIterator = contextHash.constBegin();

        while(contextIterator != contextHash.constEnd()){
            delete contextIterator.value();
            qDebug() << "Removed context: " << contextIterator.key();
            ++contextIterator;
        }

        contextHash.clear();
    }
}

QString ContextManager::newCameraContext(TriclopsDemo::CameraContext *previousContext)
{
    TriclopsDemo::CameraContext *tempContext = new TriclopsDemo::CameraContext;
    bool initResult = tempContext->Initialize();

    // This is a guard against multiple camera initialisation through different
    // instances of the application. I fear this might lead to some unwanted
    // behaviours, so I will tag it in order to remember to check it when I
    // fix the Isochronous problem.
    // FIXME
    bool sanityCheck = tempContext->CameraSanityCheck(previousContext);

    // At exit check if QString is Empty
    QString contextTag = "";

    if (initResult && sanityCheck){
        contextTag = tempContext->Identificator();
        contextHash.insert(contextTag, tempContext);
        assert(contextHash.value(contextTag) != NULL);

        qDebug() << "Initialised Context: " << contextTag;
    }

    return contextTag;
}

QString ContextManager::newOfflineContext()
{
    TriclopsDemo::OfflineContext *tempContext = new TriclopsDemo::OfflineContext;
    bool initResult = tempContext->Initialize();

    // At exit check if QString is Empty
    QString contextTag = "";

    if (initResult){
        contextTag = tempContext->Identificator();
        contextHash.insert(contextTag, tempContext);
        assert(contextHash.value(contextTag) != NULL);

        qDebug() << "Initialised Context: " << contextTag;
    }

    return contextTag;
}

TriclopsDemo::ViewContext* ContextManager::getContext(QString &contextID)
{
    return contextHash.value(contextID);
}

void ContextManager::removeContext(const QString &contextID)
{
    TriclopsDemo::ViewContext *tmpContext = contextHash.value(contextID);
    qDebug() << "Removed context: " << contextID;
    delete tmpContext;
    contextHash.remove(contextID);
}

}
