//=============================================================================
// Copyright © 2015 Point Grey Research, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Point Grey Research, Inc. (PGR).
//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: FPSCounter.cpp 275159 2016-03-30 13:58:24Z micheleb $
//=============================================================================

//=============================================================================
// System Includes
//=============================================================================
#include <algorithm>

//=============================================================================
// Project Includes
//=============================================================================
#include "FPSCounter.h"

namespace TriclopsDemo {

FPSCounter::FPSCounter(unsigned int samples, float smooth):
    sampleSize(samples), smoothFactor(smooth)
{
    sampleBuffer.reserve(samples);
    EWMA = 0.0f;
}

void FPSCounter::InsertSample(float newSample)
{
    if (sampleBuffer.size() >= sampleSize) {
        std::rotate(sampleBuffer.begin(), sampleBuffer.begin() + 1, sampleBuffer.end());
        sampleBuffer.resize(sampleSize - 1);
    }

    sampleBuffer.push_back(newSample);

    EWMA = smoothFactor * newSample + (1.0f - smoothFactor) * EWMA;
}

float FPSCounter::RollingAverage()
{
    float average = 0;
    int currentSampleSize = sampleBuffer.size();

    for (std::vector<float>::reverse_iterator i = sampleBuffer.rbegin();
            i != sampleBuffer.rend(); ++i ) {
        average += *i / currentSampleSize;
    }

    return average;
}

float FPSCounter::ExponentialRollingAverage()
{
    return EWMA;
}

} // namespace TriclopsDemo
