//=============================================================================
// Copyright © 2015 Point Grey Research, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Point Grey Research, Inc. (PGR).
//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: PointCloudData.h 275159 2016-03-30 13:58:24Z micheleb $
//=============================================================================

#ifndef POINTCLOUDDATA_H
#define POINTCLOUDDATA_H

//=============================================================================
// Project Includes
//=============================================================================
#include "TriclopsDemoDefinitions.h"
#include "StereoData.h"
#include "FrameStructures.h"

//=============================================================================
// Qt Includes
//=============================================================================
#include <QVector>
#include <QVector3D>
#include <QOpenGLFunctions>
#include <QMutex>

//=============================================================================
// PGR Includes
//=============================================================================
#include <triclops.h>

namespace TriclopsDemo {

class PointCloudData
{
public:
    PointCloudData(TriclopsDemo::StereoData &sharedContext,
                   TriclopsDemo::SharedBitmap_t &rectified);
    ~PointCloudData();

    TriclopsError UpdatePointCloud(bool bDisparityComputationRequired = false);

    void AddPointCoords(float x, float y, float z);
    void AddPointCoords(const QVector3D &newPoint);

    void AddTexCoord(const TriclopsDemo::Resolution_t &newPoint);
    void AddTexCoord(unsigned int pixelX, unsigned int pixelY);


    void UpdateResolution(const TriclopsDemo::Resolution_t &newResolution);

    void NewDisparityMap(const TriclopsDemo::Resolution_t &newResolution);

    void Lock();
    void Unlock();

    const float* PointData() const;

    int CloudSize() const;

    TriclopsDemo::SharedBitmap_t &getRectifiedTexture() const;

    QVector<GLfloat> getCameraFOV() const;

    QVector<GLfloat> getCameraBackdrop() const;

private:
    TriclopsDemo::SharedBitmap_t &rectifiedTexture;
    TriclopsDemo::StereoData &frameContext;

    TriclopsDemo::Resolution_t disparitySize;
    QVector<GLfloat> disparityMap;

    QVector<GLfloat> cameraFOV;

    QVector<GLfloat> cameraBackdrop;

    QMutex mutex;
};

} // namespace TriclopsDemo

#endif // POINTCLOUDDATA_H
