//=============================================================================
// Copyright © 2015 Point Grey Research, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Point Grey Research, Inc. (PGR).
//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: PointCloudWindow.h 275159 2016-03-30 13:58:24Z micheleb $
//=============================================================================


#ifndef POINTCLOUDWINDOW_H
#define POINTCLOUDWINDOW_H

//=============================================================================
// Project Includes
//=============================================================================
#include "FrameBitmaps.h"
#include "PointCloudData.h"

//=============================================================================
// Qt Includes
//=============================================================================
#include <QOpenGLWidget>
#include <QOpenGLFunctions>
#include <QOpenGLBuffer>
#include <QOpenGLVertexArrayObject>
#include <QColor>
#include <QOpenGLShaderProgram>
#include <QPoint>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QVector>
#include <QTime>
#include <QHash>
#include <QOpenGLTexture>
#include <QVector3D>

namespace TriclopsDemo {

class PointCloudWindow : public QOpenGLWidget, protected QOpenGLFunctions
{
    Q_OBJECT

public:
    explicit PointCloudWindow(TriclopsDemo::PointCloudData &sharedCloud,
                              QWidget *parent = 0);
    ~PointCloudWindow();

signals:
    void clicked();
    void Closing();

public slots:

    void on_NewFrameAvailable();
    void on_pixelClicked(QVector3D centre);

protected:
    void initializeGL() Q_DECL_OVERRIDE;
    void paintGL() Q_DECL_OVERRIDE;
    void resizeGL(int width, int height) Q_DECL_OVERRIDE;
    void mousePressEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void mouseMoveEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void mouseReleaseEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
    void keyReleaseEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE;

private:
    QColor clearColor;
    TriclopsDemo::PointCloudData &pointCloud;

    static const int CLOUD_VERTEX_LOCATION     = 0;
    static const int CLOUD_TEXTURE_LOCATION    = 1;

    QOpenGLShaderProgram        cloudProgram;
    QOpenGLBuffer               cloudVBO;
    QOpenGLVertexArrayObject    cloudVAO;
    QOpenGLTexture              *cloudTexture;

    QOpenGLShaderProgram        backdropProgram;
    QOpenGLBuffer               backdropVBO;
    QOpenGLVertexArrayObject    backdropVAO;

    QOpenGLShaderProgram        FOVProgram;
    QOpenGLBuffer               FOVVBO;
    QOpenGLVertexArrayObject    FOVVAO;

    float aspectRatio;
    int pointQuantity;

    QPoint lastMousePosition;
    int xRot;
    int yRot;
    int zRot;

    QHash<int, bool> downKeys;
    float xPos;
    float yPos;
    float zPos;

    bool bOrtho;

    QTime frameRenderTime;

    void Create3DObjects();
    void rotateBy(int xAngle, int yAngle, int zAngle);
    void moveCamera(int deltaTime);

    QVector3D centreOfRotation;
};

} // namespace TriclopsDemo

#endif // POINTCLOUDWINDOW_H
