//=============================================================================
// Copyright © 2015 Point Grey Research, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Point Grey Research, Inc. (PGR).
//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: RenderWindow.h 287944 2016-06-23 15:49:35Z micheleb $
//=============================================================================

#ifndef RENDERWINDOW_H
#define RENDERWINDOW_H

//=============================================================================
// Project Includes
//=============================================================================
#include "FrameBitmaps.h"
#include "FrameViewer.h"

//=============================================================================
// Qt Includes
//=============================================================================
#include <QWidget>
#include <QString>
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QPointF>
#include <QPoint>
#include <QColor>

namespace Ui {
    class RenderWindow;
}

class RenderWindow : public QWidget
{
    Q_OBJECT

public:
    explicit RenderWindow(const QString &tag,
                          TriclopsDemo::FrameBitmaps &sharedBuffer,
                          QWidget *parent = 0);
    ~RenderWindow();

    RenderWindow& SetRenderTag(QString &newTag);

    QString RenderTag() const;

    TriclopsDemo::BitmapIdentificator_t CurrentBitmap() const;

signals:
    void Closing();
    void RemoveContext(QString tag);

    void hoveringOverFeed(QPoint location, QColor color);
    void pixelClicked(QPoint location);

    void RenderedBitmapChanged(const QString &tag);

public slots:
    void renderNewFrame();
    void ChangeRenderedBitmap(TriclopsDemo::BitmapIdentificator_t newBitmap);

    void on_hoveringOverFeed(QPointF location);
    void on_pixelClicked(QPointF location);

private:
    Ui::RenderWindow *ui;
    QString renderTag;

    TriclopsDemo::FrameBitmaps &bitmaps;

    QGraphicsScene *renderingScene;
    QPixmap frameQPixmap;
    QGraphicsPixmapItem *onScreenPixmap;

    TriclopsDemo::BitmapIdentificator_t bitmapToRender;

    void on_FeedChange();

protected:
    void resizeEvent(QResizeEvent *event);

private slots:
    void on_FeedCombo_currentIndexChanged(int index);
    void on_LeftRadio_released();
    void on_SideBySideRadio_released();
    void on_RefRadio_released();
};

#endif // RENDERWINDOW_H
