//=============================================================================
// Copyright © 2015 Point Grey Research, Inc. All Rights Reserved.
//
// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with Point Grey Research, Inc. (PGR).
//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.
//=============================================================================

//=============================================================================
// $Id: UIControlsManager.cpp 287944 2016-06-23 15:49:35Z micheleb $
//=============================================================================

//=============================================================================
// Project Includes
//=============================================================================
#include "UIControlsManager.h"

namespace TriclopsDemo {

UIControlsManager::UIControlsManager():
    windowCount(0)
{

}

UIControlsManager::~UIControlsManager()
{
    DisableControls();
    menus.clear();
}

void UIControlsManager::AddControl(QWidget *newControl)
{
    menus.append(newControl);
}

void UIControlsManager::AddControl(QAction *newAction)
{
    actions.append(newAction);
}

void UIControlsManager::AddControls(const QList<QWidget *> &newControls)
{
    menus.append(newControls);
}

void UIControlsManager::AddControls(const QList<QAction *> &newControls)
{
    actions.append(newControls);
}

void UIControlsManager::IncrementWindowCount()
{
    if (windowCount == 0) {
        EnableControls();
    }

    ++windowCount;
}

void UIControlsManager::DecrementWindowCount()
{
    // No decrementing when there are no windows
    Q_ASSERT(windowCount > 0);

    --windowCount;

    if (windowCount == 0) {
        DisableControls();
    }
}

void UIControlsManager::EnableControls()
{
    int controlQty = menus.size();

    for (int i = 0; i < controlQty; ++i) {
        menus[i]->setEnabled(true);
    }

    controlQty = actions.size();

    for (int i = 0; i < controlQty; ++i) {
        actions[i]->setEnabled(true);
    }
}

void UIControlsManager::DisableControls()
{
    windowCount = 0;

    int controlQty = menus.size();

    for (int i = 0; i < controlQty; ++i) {
        menus[i]->setEnabled(false);
    }

    controlQty = actions.size();

    for (int i = 0; i < controlQty; ++i) {
        actions[i]->setEnabled(true);
    }
}

} // namespace TriclopsDemo
